!{\src2tex{textfont=tt}}
!!****f* ABINIT/atompaw2abinit
!! NAME
!! atompaw2abinit
!!
!! FUNCTION
!! Main routine for building a psp file for PAW calculations
!! in Abinit, starting from the code atomPAW (N. Holzwarth).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (GJ, FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! NOTES
!! The present main routine drives the following operations :
!!
!! 1) Read useful data from the output file of AtomPAW
!! 2) Calculate quantities that do not depend from self-consistent PAW
!!    iterations
!! 3) Write an output file to be read as an input PAW atomic data
!!    file in abinit
!!
!!       Units: Energies=Hartree, Lengths=Bohr
!!
!! Version number= 1.4.2 (august 2005)
!!
!! PARENTS
!!
!! CHILDREN
!!    calc_dij0,calc_kij,calc_hatden,calc_qijl,calc_rhoij0,calc_shapef,
!!    calc_tcore,calc_vhnzc,calc_vhtnzc,init_defs_mach,initpawps1,
!!    initpawps2,opt_proj,ortho_phi,rdpawps1,rdpawps2,wrpawps
!!
!! SOURCE

 program atompaw2abinit

 use defs_basis
 use defs_pawps

 implicit none

!==================================================================
! DECLARATIONS

 integer, parameter :: unit_atompaw(3)=(/21,22,23/),&
&                      unit_abinit=25

 integer  :: loggrd_size_def=350
 real(dp) :: loggrd_step_def=0.035d0

 real(dp) :: ecut_rso_def=20.d0,&
&            gfact_rso_def=2.d0,&
&            werror_rso_def=1.d-3

 character*(fnlen) :: empty='',&
&                     file_abinit='abinit.pawps',&
&                     file_atompaw(3)=&
&                     (/'atomicdata','density   ','potential '/)
 character*(fnlen) :: response
!These types are defined in defs_pawps
 type(loggrd_type)   :: loggrd
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pawps_type)    :: pawps
 type(pawrso_type)   :: pawrso
 type(pshead_type)   :: pshead

!******************************************************************
!BEGIN EXECUTABLE SECTION

 call Init_defs_mach
!
!------------------------------------------------------------------
!---- Write header and read file names
!------------------------------------------------------------------

 write(6,'(a)') '========================================================'
 write(6,'(a)') '==   atompaw2abinit - v1.4.2:                         =='
 write(6,'(a)') '==   Converts a PAW pseudopotential file generated    =='
 write(6,'(a)') '==   by "atompaw" code (from N. Holzwarth)            =='
 write(6,'(a)') '==   into a PAW atomic data file readable             =='
 write(6,'(a)') '==   by Abinit (v4.3.1+)                              =='
 write(6,'(a)') '==                                                    =='
 write(6,'(a)') '==   "atompaw" can be found at:  http://pwpaw.wfu.edu =='
 write(6,'(a)') '========================================================'

 write(6,'(/,3a)',ADVANCE='NO') &
&   'Input atomicdata file name (from atompaw) [',trim(file_atompaw(1)),'] ? '
 read (5,'(A)') response
 if (response/=empty) file_atompaw(1)=trim(response)

 write(6,'(3a)',ADVANCE='NO') &
&   'Input densities file name (from atompaw) [',trim(file_atompaw(2)),'] ? '
 read (5,'(A)') response
 if (response/=empty) file_atompaw(2)=trim(response)

 write(6,'(3a)',ADVANCE='NO') &
&   'Input potentials file name (from atompaw) [',trim(file_atompaw(3)),'] ? '
 read (5,'(A)') response
 if (response/=empty) file_atompaw(3)=trim(response)

 write(6,'(3a)',ADVANCE='NO') &
&   'Output psp file name (for Abinit) [',trim(file_abinit),'] ? '
 read (5,'(A)') response
 if (response/=empty) file_abinit=trim(response)
 write(6,*)

 write(6,'(a,/,a)',ADVANCE='NO') &
&   'Do you want to transfer atomic data from AtomPAW',&
&   'on a (reduced) logarithmic grid    (recommended) [y] ? '
 read (5,'(A)') response
 loggrd%uselog=(response==empty.or.response=="y".or.response=="Y")

 loggrd%meshsz=loggrd_size_def;loggrd%log_step=loggrd_step_def
 if (loggrd%uselog) then
  write(6,'(a,i3,a,f5.3,a)',ADVANCE='NO') &
&    'Logarithmic grid: Number of pts, logarithmic step [',&
&    loggrd_size_def,', ',loggrd_step_def,'] ? '
  read (5,'(A)') response
  if (response/=empty) read(unit=response,fmt=*) loggrd%meshsz,loggrd%log_step
  write(6,*)
 endif

 write(6,'(a,/,a)',ADVANCE='NO') &
&   'Do you want to improve non-local projector',&
&   'by using "Real Space Optimization" (King-Smith et al.) [y] ? '
 read (5,'(A)') response
 pawrso%userso=(response==empty.or.response=="y".or.response=="Y")

 pawrso%ecut=ecut_rso_def;pawrso%gfact=gfact_rso_def;pawrso%werror=werror_rso_def
 if (pawrso%userso) then
  write(6,'(a,f4.1,a,f3.1,a,g6.1,a)',ADVANCE='NO') &
&    'Real Space optim.: Ecut, Gamma/Gmax, Wl(error) [',&
&    ecut_rso_def,', ',gfact_rso_def,', ',werror_rso_def,'] ? '
  read (5,'(A)') response
  if (response/=empty) read(unit=response,fmt=*) pawrso%ecut,pawrso%gfact,pawrso%werror
  write(6,*)
 endif

!------------------------------------------------------------------
!---- Read PAW pseudopotential file from AtomPAW
!------------------------------------------------------------------

 allocate(pshead%orbitals(20))

!---- Read the header (sizes of arrays)
 call rdpawps1(file_atompaw,pshead,unit_atompaw)

!---- Initialize some useful data
 allocate(pawdata%indlmn(6,pshead%lmn_size))
 call initpawps1(pawdata,pshead)

!---- Allocate memory
 allocate(pawps%phi(pshead%mesh_size,pshead%basis_size))
 allocate(pawps%tphi(pshead%mesh_size,pshead%basis_size))
 allocate(pawps%tproj(pshead%proj_meshsz,pshead%basis_size))
 allocate(pawps%coredens(pshead%bigmesh_size))
 allocate(pawps%tcoredens(pshead%bigmesh_size))
 allocate(pawps%dij0(pshead%lmn2_size))
 allocate(pawps%rhoij0(pshead%lmn2_size))
 allocate(pawps%vhtnzc(pshead%bigmesh_size))
 allocate(pawarray%shapefunc(pshead%mesh_size))
 allocate(pawarray%gnorm(pshead%l_size))
 allocate(pawarray%hatden4pr2(pshead%mesh_size))
 allocate(pawarray%q00hat(pshead%mesh_size))
 allocate(pawarray%qijl(pshead%lmn2_size,pshead%l_size**2))
 allocate(pawarray%kij(pshead%lmn2_size))
 allocate(pawarray%coreden4pr2(pshead%bigmesh_size))
 allocate(pawarray%tcoreden4pr2(pshead%bigmesh_size))
 allocate(pawarray%valeden4pr2(pshead%bigmesh_size))
 allocate(pawarray%tvaleden4pr2(pshead%bigmesh_size))
 allocate(pawarray%rveff(pshead%bigmesh_size))
 allocate(pawarray%vhnzc(pshead%bigmesh_size))
 allocate(pawarray%vhnzcij(pshead%lmn2_size))
 allocate(pawarray%vhtnzcij(pshead%lmn2_size))
 allocate(pawdata%rad(pshead%bigmesh_size))

!---- Initialize some useful data
 call initpawps2(pawdata,pshead)

!---- Read the body
 call rdpawps2(file_atompaw,pawarray,pawdata,pawps,pshead,unit_atompaw)
!------------------------------------------------------------------
!---- Compute new data needed by Abinit
!------------------------------------------------------------------
 call calc_shapef(pawarray,pawdata,pshead)
!      Re-orthogonalization of Phi, tPhi, tProj
       call ortho_phi(pawarray,pawps,pshead)
 call calc_hatden(pawarray,pawdata,pshead)
 call calc_tcore(pawarray,pawdata,pawps,pshead)
 call calc_vhnzc(pawarray,pawdata,pawps,pshead)
 call calc_vhtnzc(pawarray,pawdata,pawps,pshead)
 call calc_qijl(pawarray,pawdata,pawps,pshead)
 call calc_kij(pawarray,pawdata,pawps,pshead)
 call calc_dij0(pawarray,pawdata,pawps,pshead)
 call calc_rhoij0(pawarray,pawdata,pawps,pshead)
 call opt_proj(pawps,pawrso,pshead)

!------------------------------------------------------------------
!---- Write PAW pseudopotential file for Abinit
!------------------------------------------------------------------

 call wrpawps(file_abinit,loggrd,pawps,pawrso,pshead,unit_abinit)

!------------------------------------------------------------------
!---- Deallocate memory
!------------------------------------------------------------------

 deallocate(pshead%orbitals)
 deallocate(pawps%phi,pawps%tphi,pawps%tproj)
 deallocate(pawps%coredens,pawps%tcoredens)
 deallocate(pawps%dij0,pawps%rhoij0,pawps%vhtnzc)
 deallocate(pawarray%gnorm,pawarray%shapefunc)
 deallocate(pawarray%hatden4pr2,pawarray%valeden4pr2,pawarray%tvaleden4pr2)
 deallocate(pawarray%q00hat,pawarray%qijl,pawarray%kij,pawarray%coreden4pr2)
 deallocate(pawarray%vhnzc,pawarray%vhnzcij,pawarray%vhtnzcij)
 deallocate(pawarray%rveff)
 deallocate(pawdata%indlmn,pawdata%rad)

 write(6,'(/,a)') 'Program ended.'

 end program
!!***
