!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_kij
!! NAME
!! calc_kij
!! 
!! FUNCTION
!! Compute kij terms:
!!  kij=<phi_i|T|phi_j>-<tphi_i|T|tphi_j>
!!  where T is the kinetic operator
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!! 
!! INPUTS 
!!  pawdata
!!    %indlmn(6,lmn_size)= Gives l,m,n,ln,lm,s for i=lmn
!!    %rad(bigmesh_size)= Coordinates of the radial grid
!!  pawps
!!    %phi(mesh_size,basis_size)= PAW atomic wavefunctions
!!                                on the radial grid
!!    %tphi(mesh_size,basis_size)= PAW atomic pseudo-wavefunctions
!!                                 on the radial grid
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %lmn_size= Number of elements for the paw basis
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %rad_step= Step corresponding to radial mesh
!!
!! OUTPUT
!!  pawarray
!!    %kij(lmn2_size)= Kinetic overlap operator
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDRENN
!!      ctrap,nderiv
!!
!! SOURCE

 subroutine calc_kij(pawarray,pawdata,pawps,pshead)

 use defs_basis
 use defs_pawps

 implicit none
 
!Arguments ---------------------------------------------
!These types are defined in defs_pawps
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer :: ib,ilm,ilmn,iln,ir,jlm,jlmn,jln,j0lmn,klmn,llp
 real(dp) :: intg,tintg
 real(dp), allocatable :: chi(:,:),ff(:),tchi(:,:),tff(:),&
&                         y1(:),y2(:),ty1(:),ty2(:)

!--------------------------------------------------------

!First, compute |chi_i>=T|phi_i>
!---------------------------------
 allocate(chi (pshead%mesh_size,pshead%basis_size))
 allocate(tchi(pshead%mesh_size,pshead%basis_size))
 allocate(y1(pshead%mesh_size-1),ty1(pshead%mesh_size-1))
 allocate(y2(pshead%mesh_size-1),ty2(pshead%mesh_size-1))
 do ib=1,pshead%basis_size
  llp=pshead%orbitals(ib)*(pshead%orbitals(ib)+1)
  y2=zero;ty2=zero
  call nderiv(pshead%rad_step,pawps%phi(2:pshead%mesh_size,ib),&
&             y1,pshead%mesh_size-1)
  call nderiv(pshead%rad_step,y1,y2,pshead%mesh_size-1)
  call nderiv(pshead%rad_step,pawps%tphi(2:pshead%mesh_size,ib),&
&             ty1,pshead%mesh_size-1)
  call nderiv(pshead%rad_step,ty1,ty2,pshead%mesh_size-1)
  chi(1,ib)=zero;tchi(1,ib)=zero
  do ir=2,pshead%mesh_size
   chi (ir,ib)=y2 (ir-1)-(dble(llp)/pawdata%rad(ir)**2)*pawps%phi (ir,ib)
   tchi(ir,ib)=ty2(ir-1)-(dble(llp)/pawdata%rad(ir)**2)*pawps%tphi(ir,ib)
  enddo
 enddo
 deallocate(y1,ty1,y2,ty2)

!Now, compute calculate kij=<phi_i|T|phi_j>
!------------------------------------------
 allocate(ff(pshead%mesh_size),tff(pshead%mesh_size))
 pawarray%kij=zero
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jlm=pawdata%indlmn(4,jlmn);jln=pawdata%indlmn(5,jlmn)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   ilm=pawdata%indlmn(4,ilmn);iln=pawdata%indlmn(5,ilmn)
   if (jlm==ilm) then
    ff(:)=pawps%phi(:,iln)*chi(:,jln)
    call ctrap(pshead%mesh_size,ff,pshead%rad_step,intg)
    tff(:)=pawps%tphi(:,iln)*tchi(:,jln)
    call ctrap(pshead%mesh_size,tff,pshead%rad_step,tintg)
    pawarray%kij(klmn)=tintg-intg
   endif
  enddo
 enddo
 deallocate(ff,tff)

 deallocate(chi,tchi)

 pawarray%kij=pawarray%kij*0.5d0   ! Translate from Rydberg to Hartree

 end subroutine
!!***
