!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_vhtnzc
!! NAME
!! calc_vhtnzc
!! 
!! FUNCTION
!! Compute vh(tnZc) and the <phi_i|vh(tnZc)|phi_j> matrix elements
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawarray
!!    %coreden4pr2(bigmesh_size)= Core density multiplied by 4Pi.r2
!!    %hatden4pr2(mesh_size)= Compensation charge density
!!    %rveff(bigmesh_size)= Effective all electron atomic potential*r
!!    %tvaleden4pr2(bigmesh_size) = Density 4*pi*r^2*tn1,
!!                                  coming from the code Atompaw
!!    %valeden4pr2(bigmesh_size) = Density 4*pi*r^2*n1,
!!                                 coming from the code Atompaw
!!  pawdata
!!    %indlmn(6,lmn_size)= Gives l,m,n,ln,lm,s for i=lmn
!!    %rad(bigmesh_size)= Coordinates of the radial grid
!!  pawps
!!    %tphi(mesh_size,basis_size)= PAW atomic pseudo-wavefunctions
!!                                 on the radial grid
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %bigmesh_size= Dimension of radial mesh for read coredens/valedens
!!    %core_charge= Core charge
!!    %lmn_size= Number of elements for the paw basis
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %pspxc_atom= AtomPaw s code number for the exchange-correlation
!!    %rad_step= Step corresponding to radial mesh
!!    %rc_sph= Default PAW sphere radius
!!
!! OUTPUT
!!  pawarray
!!    %vhtnzcij(lmn2_size)= matrix element of the Hartree potential of
!!                          the ps-density of the nucleus+core electrons
!!  pawps
!!    %vhtnzc(bigmesh_size)= Hartree potential of the ps-density
!!                           of the nucleus + core electrons
!!  pshead
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhnzc
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!      ctrap,exchcor,gaups,poisson,sinps
!!
!! SOURCE

 subroutine calc_vhtnzc(pawarray,pawdata,pawps,pshead)

  use defs_pawps
  use defs_basis

  implicit none

!Arguments ---------------------------------------------
!These types are defined in defs_pawps
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer, parameter :: option=1

 integer :: ilm,ilmn,iln,ir,jlm,jlmn,jln,j0lmn,klmn,nc1
 real(dp) :: etxc,eexc,intg_diff,qcore,test_vhtnzc,yp1,yp2,yp3
 real(dp),allocatable :: den_1hat(:),den_1hatc(:),den_tzc(:),ff(:),&
&                        veff(:),tveff(:),rvh(:),rvxc(:)

!-------------------------------------------------------

!We have three possibility for the computation of VH(tnzc):
! 1- Directly, without pseudization
! 2- From Veff, using a gaussian pseudization
! 3- From Veff, using a "sin(x)/x" pseudization
! The choice is governed by the "option" variable.

!==================================================================
! First option for VH(tnzc) : directly, without pseudization
!==================================================================
 if (option==1) then
! Compute Zcore
  allocate(den_tzc(pshead%bigmesh_size),rvh(pshead%bigmesh_size))
  den_tzc(1:pshead%mesh_size)=pawarray%coreden4pr2 (1:pshead%mesh_size) &
&                            -pawarray%tcoreden4pr2(1:pshead%mesh_size)
  call ctrap(pshead%mesh_size,den_tzc,pshead%rad_step,qcore)
! Compute tnZC(r)
  den_tzc(1:pshead%mesh_size)=pawarray%tcoreden4pr2(1:pshead%mesh_size) &
&                            +pawarray%q00hat(1:pshead%mesh_size)*(qcore-pshead%atomic_charge)
  den_tzc(pshead%mesh_size+1:pshead%bigmesh_size)=pawarray%coreden4pr2(pshead%mesh_size+1:pshead%bigmesh_size)
! Compute r*Vh(tnZC(r)) using poisson equation
  rvh(1)=zero
  call poisson(den_tzc(2:pshead%bigmesh_size),pshead%rad_step,&
&              pshead%bigmesh_size-1,&
&             -pshead%atomic_charge+pshead%core_charge,&
&              rvh(2:pshead%bigmesh_size))
! Compute vhtnZc from r*vh
  pawps%vhtnzc(2:pshead%bigmesh_size)=rvh(2:pshead%bigmesh_size) &
&                                    /pawdata%rad(2:pshead%bigmesh_size)
  deallocate(den_tzc,rvh)
! Deduce vhtnZc at r=0
  yp2=(pawps%vhtnzc(3)-pawps%vhtnzc(2))/(pawdata%rad(3)-pawdata%rad(2))
  yp3=(pawps%vhtnzc(4)-pawps%vhtnzc(3))/(pawdata%rad(4)-pawdata%rad(3))
  yp1=yp2+(yp2-yp3)*(pawdata%rad(2)-pawdata%rad(1))/(pawdata%rad(3)-pawdata%rad(2))
  pawps%vhtnzc(1)=pawps%vhtnzc(2)-(yp1+yp2)*pawdata%rad(2)

!==================================================================
! 2nd and Third options for VH(tnzc) : from Veff,
!                   - using a gaussian pseudization (option=2)
!                   - using a "sin(x)/x" pseudization (option=3)
!==================================================================
 elseif (option==2.or.option==3) then
! Get veff from r*veff
  allocate(veff(pshead%bigmesh_size),tveff(pshead%bigmesh_size),&
&          den_1hat(pshead%bigmesh_size),den_1hatc(pshead%bigmesh_size),&
&          rvh(pshead%bigmesh_size),rvxc(pshead%bigmesh_size))
  do ir=2,pshead%bigmesh_size
   veff(ir) = pawarray%rveff(ir)/pawdata%rad(ir)
  enddo
! Deduce veff at r=0
  yp2=(veff(3)-veff(2))/(pawdata%rad(3)-pawdata%rad(2))
  yp3=(veff(4)-veff(3))/(pawdata%rad(4)-pawdata%rad(3))
  yp1=yp2+(yp2-yp3)*(pawdata%rad(2)-pawdata%rad(1))/(pawdata%rad(3)-pawdata%rad(2))
  veff(1)=veff(2)-(yp1+yp2)*pawdata%rad(2)
! Compute the pseudopotential tveff from veff
!              tveff(r)=A*exp(Br^2)  if r<rc1 -- option=2
!              tveff(r)=A*sin(Q.r)/r if r<rc1 -- option=3
  nc1=int(pshead%rc_sph/pshead%rad_step/1.2d0)+1
  if     (option==2) then
   call gaups(veff,tveff,pshead%rad_step,nc1)
  elseif (option==3) then
   call sinps(veff,tveff,pshead%rad_step,nc1)
 endif
  tveff(nc1+1:pshead%bigmesh_size) = veff(nc1+1:pshead%bigmesh_size)
! Compute: den_1hat(r) =4*pi*r^2*(tn1(r)+nhat(r))
!          den_1hatc(r)=4*pi*r^2*(tn1(r)+nhat(r)+tnc(r))
  do ir=1,pshead%mesh_size
   den_1hat(ir)=pawarray%tvaleden4pr2(ir)+pawarray%hatden4pr2(ir)
   den_1hatc(ir)=den_1hat(ir)+pawarray%tcoreden4pr2(ir)
  enddo
  do ir=pshead%mesh_size+1,pshead%bigmesh_size
   den_1hat(ir) =pawarray%valeden4pr2(ir)
   den_1hatc(ir)=den_1hat(ir)+pawarray%coreden4pr2(ir)
  enddo
! Compute r*Vh(tn1+nhat) using poisson equation
  rvh(1)=zero
  call poisson(den_1hat(2:pshead%bigmesh_size),pshead%rad_step,&
&              pshead%bigmesh_size-1,&
&              pshead%atomic_charge-pshead%core_charge,&
&              rvh(2:pshead%bigmesh_size))
! Compute r*Vxc(tn1+nhat+tnc)
  call exchcor(den_1hatc,eexc,etxc,pshead%rad_step,pshead%pspxc_atom,&
&              pshead%bigmesh_size,rvxc)
! Compute vhtnZc(r)=tveff(r)-Vh(tn1+nhat)-Vxc(tn1+nhat+tnc)
  do ir=2,pshead%bigmesh_size
   pawps%vhtnzc(ir)= tveff(ir)-(rvh(ir)+rvxc(ir))/pawdata%rad(ir)
  enddo
  deallocate(den_1hat,den_1hatc,veff,tveff,rvh,rvxc)
! Deduce vhtnZc at r=0
  yp2=(pawps%vhtnzc(3)-pawps%vhtnzc(2))/(pawdata%rad(3)-pawdata%rad(2))
  yp3=(pawps%vhtnzc(4)-pawps%vhtnzc(3))/(pawdata%rad(4)-pawdata%rad(3))
  yp1=yp2+(yp2-yp3)*(pawdata%rad(2)-pawdata%rad(1))/(pawdata%rad(3)-pawdata%rad(2))
  pawps%vhtnzc(1)=pawps%vhtnzc(2)-(yp1+yp2)*pawdata%rad(2)
 endif

!==================================================================
! End of options
!==================================================================

!Compute matrix elements: vhtnZc_ij(r)=<tphi_i|vh(tnZc)|thpi_j>
!------------------------------------------------------------------
 allocate(ff(pshead%mesh_size))
 pawarray%vhtnzcij=zero
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jlm=pawdata%indlmn(4,jlmn);jln=pawdata%indlmn(5,jlmn)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   ilm=pawdata%indlmn(4,ilmn);iln=pawdata%indlmn(5,ilmn)
   if (jlm==ilm) then
    do ir=1,pshead%mesh_size
     ff(ir)=pawps%tphi(ir,iln)*pawps%vhtnzc(ir)*pawps%tphi(ir,jln)
    enddo
    call ctrap(pshead%mesh_size,ff,pshead%rad_step,&
&              pawarray%vhtnzcij(klmn))
   endif
  enddo
 enddo
 deallocate(ff)

!Test the behaviour of VhtnZc at long range: vhnzc(r)=-Z/r ?
!-----------------------------------------------------------
  write(6,'(/,a)') 'Info:'
  write(6,'(a,i4,a,a,g11.4)') &
&   '  At r_vloc=r(',pshead%vloc_meshsz,'),',&
&   ' VHartree(ntild(Zv+Zc))= -Zv/r + ',&
&     pawps%vhtnzc(pshead%vloc_meshsz) &
&    +(pshead%atomic_charge-pshead%core_charge)/pawdata%rad(pshead%vloc_meshsz)
  write(6,'(a)') &
&   '  This quantity must be as small as possible.'
 
 end subroutine calc_vhtnzc




!============================================
 subroutine gaups(ff,tff,hh,nn)

  use defs_basis

  implicit none

  integer :: nn
  real(dp) :: hh,ff(nn),tff(nn)

  integer :: ii
  real(dp) :: f0,f0p,r0

  f0=ff(nn-1)
  f0p=(ff(nn)-ff(nn-2))/(2.d0*hh)
  r0=(nn-2)*hh

  do ii=1,nn-1
   tff(ii)=f0/exp(0.5d0*f0p*r0/f0) &
&            *exp(0.5d0*f0p*(dble(ii-1)*hh)**2/(r0*f0))
  enddo

  tff(nn)=ff(nn)

 end subroutine gaups


!============================================
 subroutine sinps(ff,tff,hh,nn)

  use defs_basis

  implicit none

  integer :: nn
  real(dp) :: hh,ff(nn),tff(nn)

  integer :: ii
  real(dp) :: aa,aa1,aa2,bb,f0,f0p,gg,r0

  f0=ff(nn-1)
  f0p=(ff(nn)-ff(nn-2))/(2.d0*hh)
  r0=(nn-2)*hh

  aa1=(pi/2.d0/r0)+1.d-8
  aa2=+(3.d0*pi/2.d0/r0)-1.d-8
  gg=1.d10
  do while (abs(gg)>1.d-10)
   aa=(aa1+aa2)/2.d0
   gg=tan(aa*r0)-f0*aa/(f0p+(f0/r0))
   if (gg>0.d0) then
    aa2=aa
   else
    aa1=aa
   endif
  enddo

  bb=f0*r0/sin(aa*r0)

  do ii=2,nn-1
   tff(ii)=bb*sin(aa*dble(ii-1)*hh)/(dble(ii-1)*hh)
  enddo
  tff(1)=aa*bb
  tff(nn)=ff(nn)

 end subroutine sinps

!!***

