!{\src2tex{textfont=tt}}
!!****f* ABINIT/defs_pawps
!! NAME
!! defs_pawps
!!
!! FUNCTION
!! This module contains the following structured data types:
!!   pawps_type   : Specific paw variables
!!   pshead_type  : Abinit psp file header variables
!!   pawdata_type : Temporary data
!!   pawarray_type: Various arrays used in paw atomic data generation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! NOTES
!!
!! The paw wavefunctions tphi and phi, and the projectors tprojector radial
!! parts are defined as: f(r_vector)=f(r)/r* Ylm(r_vector)
!!
!!
!! SOURCE

 module defs_pawps

 use defs_basis
 
 implicit none


!Structures
!**********************************************************************

 type pawps_type

! ===== Real (double precision) arrays

  real(dp), pointer :: phi(:,:)
   ! phi(mesh_size,basis_size)
   ! Gives, on the radial grid, the PAW atomic wavefunctions

  real(dp), pointer :: tphi(:,:)
   ! tphi(mesh_size,basis_size)
   ! Gives, on the radial grid, the PAW atomic pseudowavefunctions

  real(dp), pointer :: tproj(:,:)
   ! tproj(proj_meshsz,basis_size)
   ! Gives, on the radial grid, the PAW projectors for each basis function

  real(dp), pointer :: coredens(:)
   ! coredens(bigmesh_size)
   ! Gives the core density of the atom

  real(dp), pointer :: tcoredens(:)
   ! tcoredens(bigmesh_size)
   ! Gives the pseudo core density of the atom

  real(dp), pointer :: dij0(:)
   ! dij0(lmn2_size)
   ! Part of the Dij term (non-local operator) completely
   !  calculated in the pseudopotential part

  real(dp), pointer :: rhoij0(:)
   ! rhoij0(lmn2_size)
   ! Atomic initialization of rhoij

  real(dp), pointer :: vhtnzc(:)
   ! vhtnzc(bigmesh_size)
   ! Gives the Hartree potential of the pseudo density
   !  of the nucleus + core electrons of the atom 

 end type pawps_type

! =====================================================================

 type pshead_type

! ===== Integer scalars

  integer :: basis_size
   ! Number of elements for the paw nl basis

  integer :: bigmesh_size
   ! Dimension of radial mesh for read coredens, valedens, tvaledens
   ! Must be greater than mesh_size

  integer :: core_meshsz
   !Dimension of radial mesh for (t)coredens for coredens<pgiven prec

  integer :: creatorid
   ! ID of psp generator (here creatorID=1 !)

  integer :: l_size
   ! Maximum value of l+1 leading to a non zero Gaunt coefficient

  integer :: lambda
   ! Lambda in gaussian type g(r)

  integer :: lmax
   ! Maximum value of l 

  integer :: lmn_size
   ! Number of elements for the paw basis

  integer :: lmn2_size
   ! lmn2_size=lmn_size*(lmn_size+1)/2

  integer :: mesh_size
   ! Dimension of radial mesh for phi, tphi ...

  integer :: mesh_type
   ! Flag: 1 if all meshes are regular grids 
   !       2 if all meshes are log. grids 

  integer :: proj_meshsz
   ! Dimension of radial mesh for tproj

  integer :: pspcod
   ! Psp code number for Abinit (here PAW->pspcod=7 !)

  integer :: pspxc_abinit
   ! Abinit s code number for the exchange-correlation functionnal

  integer :: pspxc_atom
   ! AtomPaw s code number for the exchange-correlation functionnal

  integer :: shape_type
   ! Shape function type
   ! shape_type=1 ; g(r)=sin
   ! shape_type=2 ; g(r)=exp
   ! shape_type=3 ; g(r)=tabulated

  integer :: vloc_meshsz
   ! Dimension of radial mesh for vloc=vhnzc
   ! For r>r(vloc_meshsz), we assume vloz=-Z/r

! ===== Integer arrays

   integer, pointer :: orbitals(:)
   ! orbitals(basis_size)
   ! Gives the quantum number l for each basis function

! ===== Real (double precision) scalars

  real(dp) :: atomic_charge
   ! Total atomic charge

  real(dp) :: core_charge
   ! Core charge

  real(dp) :: rad_step
   ! Step corresponding to radial mesh for phi, tphi, ...

  real(dp) :: rc_sph
   ! Default PAW sphere radius

  real(dp) :: sigma
   ! Sigma for gaussian type g(r)

! ===== Strings

  character*(fnlen) :: title
   ! Title for pseudopotential

 end type pshead_type

! =====================================================================

 type pawdata_type

! ===== Integer arrays

   integer, pointer :: indlmn(:,:)
   ! indlmn(6,lmn_size)
   ! Gives l,m,n,ln,lm,s for i=lmn

! ===== Real (double precision) arrays

  real(dp), pointer :: rad(:)
   ! rad(bigmesh_size)
   ! Coordinates of all the points of the radial grid

 end type pawdata_type

! =====================================================================

 type pawarray_type

! ===== Real (double precision) arrays

  real(dp), pointer :: coreden4pr2(:)
   ! coreden4pr2(bigmesh_size)
   ! Gives the core density multiplied by 4Pi.r2 (read from AtomPaw)

  real(dp), pointer :: gnorm(:)
   ! gnorm(l_size)
   ! Gives the normalisation of qijl for each l

  real(dp), pointer :: hatden4pr2(:)
   ! hatden4pr2(mesh_size)
   ! Compensation charge density

  real(dp), pointer :: kij(:)
   ! kij(lmn2_size)
   ! Kinetic overlap operator

  real(dp), pointer :: q00hat(:)
   ! q00hat(mesh_size)
   ! Qij_hat terms with i,j=(0,0)

  real(dp), pointer :: qijl(:,:)
   ! qijl(lmn2_size,l_size**2)
   ! q_ijl terms

  real(dp), pointer :: rveff(:)
   ! rveff(bigmesh_size)
   ! Gives the effective all electron atomic potential*r

  real(dp), pointer :: shapefunc(:)
   ! shapefunc(mesh_size)
   ! Gives the normalized shape function of the atom used for
   ! the compensation density

  real(dp), pointer :: tcoreden4pr2(:)
   ! coreden4pr2(bigmesh_size)
   ! Gives the tcore density multiplied by 4Pi.r2

  real(dp), pointer :: tvaleden4pr2(:)
   ! tvaleden4pr2(bigmesh_size)
   ! Density 4*pi*r^2*tn1, coming from the code Atompaw
   !                       from N. Holzwarth

  real(dp), pointer :: valeden4pr2(:)
   ! valeden4pr2(bigmesh_size)
   ! Density 4*pi*r^2*n1, coming from the code Atompaw
   !                      from N. Holzwarth

  real(dp), pointer :: vhnzcij(:)
   ! vhnzcij(lmn2_size)
   ! (i,j) channel of the hartree term inside
   ! the spheres for the density nZc

  real(dp), pointer :: vhnzc(:)
   ! vhnzc(bigmesh_size)
   ! Gives the Hartree potential of the density
   !  of the nucleus + core electrons of the atom

  real(dp), pointer :: vhtnzcij(:)
   ! vhtnzcij(lmn2_size)
   ! Gives the matrix element of the Hartree potential
   ! of the pseudodensity of the nucleus+core electrons of the atom

 end type pawarray_type

! =====================================================================

 type pawrso_type

! ===== Logical

 logical :: userso
  ! TRUE if Real Space Optimization is required

! ===== Real (double precision)

 real(dp) :: ecut
  ! Real Space Optimization parameter: plane wave cutoff = 1/2 Gmax**2

 real(dp) :: gfact
  ! Real Space Optimization parameter: Gamma/Gmax ratio

 real(dp) :: werror
  ! Real Space Optimization parameter: max. error W_l allowed

 end type pawrso_type

! =====================================================================

 type loggrd_type

! ===== Integer

 integer :: meshsz
  ! Mesh size for the logarithmic grid

! ===== Logical

 logical :: uselog
  ! TRUE if data are transfered on a log. grid before being written

! ===== Real (double precision)

 real(dp) :: log_step
  ! Logarithmic step for the logarithmic grid

 real(dp) :: rad_step
  ! Radial step for the logarithmic grid

 end type loggrd_type

!**********************************************************************

 end module defs_pawps
!!***
