!{\src2tex{textfont=tt}}
!!****f* ABINIT/exchcor_atom
!! NAME
!! exchcor_atom
!!
!! FUNCTION
!! This file contains a set of subroutines and functions
!! related to the exchange-correlation functionnals
!! These routine were directly extracted from the AtomPaw
!! code from N. Holzwarth
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (NH, FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! SOURCE


!*******************************************************************
!*******************************************************************
! Subroutine exchcor:
! ===================
! Calculate exchange-correlation potentials and energys
!  for density functional theory from electron density
! INPUTS
!  den(n)= electron density * (4*pi*r**2)
!  h=radial step
!  itype= exchange-correlation type
!         if itype=15 (gga case),
!         need to calculate additional functions:
!          gd=abs(grad(density))/density
!          ggd=laplacian(density)/density
!          gggd=grad(density)*grad(abs(grad(density)))/density**2
!  n= radial mesh size
! OUTPUTS
!  eexc= total exchange energy (int(den*exc))
!  etxc= eexc - int(den*vxc)
!  rvxc(n)= returned as vxc * r to the total energy
!  For r<r0 (fixed in parameter statement), replace density by Gaussian
!  form so that Laplacian does not diverge
!*******************************************************************
!*******************************************************************
 subroutine exchcor(den,eexc,etxc,h,itype,n,rvxc)

 use defs_basis
 
 implicit none
 
!Arguments ---------------------------------------------
 integer :: itype,n
 real(dp) :: eexc,etxc,h
 real(dp) :: den(n),rvxc(n)
 
!Local variables ---------------------------------------
 real(dp), parameter :: r0=0.01d0

 integer :: i,i0
 real(dp) :: bb,br,br2,cr2,cr3,cr4,exc,gd,ggd,gggd,&
&            r,r00,r2,rho,rinttmpv,t1,t2,vxc,x,x2,x3,x4
 real(dp) :: ddexp,ddlog
 real(dp) :: a(n),b(n),c(n)
 real(dp), allocatable ::  tmpr(:),tmpd(:),tmpv(:)
 
!-------------------------------------------------------

 i0=r0/h;i0=max(i0,2)
 r00=i0*h

!Exchange-correlation contribution  exc-vxc
!==========================================
  
 if (itype.eq.16) then
!-------- PBE form -----------
  allocate(tmpr(n),tmpd(n),tmpv(n))
  do i=1,n-1
   tmpr(i+1)=i*h
   tmpd(i+1)=den(i+1)/(four_pi*(i*h*i*h))
  enddo
  tmpr(1)=0
  tmpd(1)=2*tmpd(2)-tmpd(3)
  call radialexcpbe(n,tmpr,tmpd,eexc,tmpv)
  do i=1,n
   rvxc(i)=tmpv(i)*tmpr(i)
   tmpv(i)=tmpv(i)*den(i)
  enddo
  call ctrap(n,tmpv,h,rinttmpv)
  etxc=eexc-rinttmpv
  deallocate(tmpr,tmpd,tmpv)
  return

 else if (itype.lt.14) then
  a(1)=zero
  b(1)=zero
  rvxc(1)=zero
  do i=2,n
   r=(i-1)*h
   r2=r*r
   rho=den(i)/(four_pi*r2)
   call excor(rho,vxc,exc,.false.,itype)
   a(i)=den(i)*(exc-vxc)
   b(i)=den(i)*exc
   rvxc(i)=r*vxc
  enddo

 else if (itype.eq.14) then
!-------- Perdew-Wang LDA form -----------
  a(1)=zero
  b(1)=zero
  rvxc(1)=zero
  do i=2,n
   r=(i-1)*h
   r2=r*r
   rho=den(i)/(four_pi*r2)
   call pwldafunc(rho,exc,vxc)
   a(i)=den(i)*(exc-vxc)
   b(i)=den(i)*exc
   rvxc(i)=r*vxc
  enddo

 else if (itype.eq.15) then
  do i=1,n
   r=i*h
   r2=r*r
   a(i)=den(i)/(four_pi*r2)
   b(i)=ddlog(a(i))
  enddo
  call nderiv(h,b,c,n)
  call nderiv(h,c,rvxc,n)
! Replace values for r<r0 (i<i0) by Gaussian form
  t1=c(i0)/r00
  t2=rvxc(i0)
  bb=-0.75d0*t1+0.25*t2
  cr2=0.125d0*(t2-t1)
  br=bb*r00
  cr3=cr2*r00
  br2=br*r00
  cr4=cr3*r00
  do i=1,i0
   x=i
   x=x/i0
   x2=x*x
   x3=x*x2
   x4=x2*x2
   a(i)=a(i0)*ddexp(-br2*(x2-1.d0)+cr4*(x4-1.d0))
   c(i)=-2*br*x+4*cr3*x3
   rvxc(i)=-2*bb+12*cr2*x2
  enddo
  do i=1,n
   r=i*h
!  Laplacian term:
   ggd=2*c(i)/r+rvxc(i)+c(i)**2
!  Double derivative term:
   gggd=c(i)*(rvxc(i)+c(i)**2)*dsign(1.d0,c(i))
!  Gradient term:
   gd=abs(c(i))
   call excorp(a(i),gd,ggd,gggd,vxc,exc,.false.,itype)
   a(i)=den(i)*(exc-vxc)
   b(i)=den(i)*exc
   rvxc(i)=r*vxc
  enddo
 endif

!Exchange-correlation contribution to the potential
!==================================================
 call ctrap(n,a,h,etxc)
 call ctrap(n,b,h,eexc)

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine radialexcpbe:
!   Density(:) input on a uniform radial mesh of npts
!   R(:) input mesh points
!   Exc - output integrated exchange correlation energy -- in Hartree units
!   vxc(:) -- output exchange correlation potential     -- in Hartree units
!*******************************************************************
!*******************************************************************
 subroutine radialexcpbe(npts,r,density,exc,vxc)

 use defs_basis

 implicit none

!Arguments ---------------------------------------------
 integer, intent(IN) :: npts
 real(dp), intent(IN) :: r(npts),density(npts)
 real(dp), intent(OUT) :: exc, vxc(npts)

!Local variables ---------------------------------------
 integer :: i,j 
 real(dp) :: dfxcdn,dfxcdg,h,sgn
 real(dp) :: fxc(npts),gradient(npts),gradmag(npts),gxc(npts),dgxcdr(npts)

!-------------------------------------------------------

 h=r(2)-r(1)
 call nderiv(h,density,gradient,npts)
 gradmag=abs(gradient)

 do i=2,npts
  call  pbefunc(density(i),gradmag(i),fxc(i),dfxcdn,dfxcdg)
  sgn=1
  if(gradient(i)<zero) sgn=-1
  vxc(i)=dfxcdn
  gxc(i)=dfxcdg*sgn
 enddo

 call nderiv(h,gxc,dgxcdr,npts)

 do i=1,npts
  fxc(i)=fxc(i)*4*pi*(r(i)*r(i))
  vxc(i)=vxc(i)-dgxcdr(i)
 enddo

 call ctrap(npts,fxc,h,Exc)

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine pwldafunc:
! Subroutine to calculate the LDA exchange correlation functionals
! using the form of Perdew and Wang (PRB 45, 13244 (1992)
! assuming no spin polarization.
! Inside this routine, energies are in Hartree units
!*******************************************************************
!*******************************************************************
 subroutine pwldafunc(den,exc,vxc)

 use defs_basis
 use defs_excor
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp), intent(IN) :: den    !density
 real(dp), intent(OUT) :: exc,vxc

!Local variables ---------------------------------------
 real*8 :: n,g,kf,rs,ks  ! Variables depending on den
 real*8 :: decdrs,ex,ec,Logofterm,pprs
 real*8 :: term

!-------------------------------------------------------

 n=den
 if (n < machine_zero)  then
  exc=0.d0; vxc=0.d0
  return
 endIf

 kf=(3.d0*(pi**2)*n)**0.3333333333333333333333333333d0
 rs=(3.d0/(4.d0*pi*n))**0.3333333333333333333333333333d0
 ks=sqrt(4.d0*kf/pi)

 ex=-3.d0*kf/(4.d0*pi)
 pprs=sqrt(rs)*(b1+b3*rs)+rs*(b2+b4*rs)
 term=Logofterm(1.d0/(2.d0*AA*pprs))
 ec=-2.d0*AA*(1.d0+a1*rs)*term

 exc=ex+ec

 decdrs=-(2.d0*AA*a1)*term &
   +((1.d0+a1*rs)*((b1+3*b3*rs)/(2.d0*sqrt(rs))+b2+2*b4*rs))/(pprs*(pprs+1.d0/(2.d0*AA)))
 
 vxc = (4.d0/3.d0)*ex+ec-(decdrs*rs)/3.d0

 if ((abs(exc).gt.1.d65).or.(abs(vxc).gt.1.d65)) then
  write(6,*) 'Problem in PW',n,rs,ec
 endif

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine pbefunc:
! Subroutine to calculate the exchange correlation functionals
! using the form of Perdew, Burke, and Ernzerhof (PRL 77, 3865 (1996))
! assuming no spin polarization.
! Inside this routine, energies are in Hartree units
!*******************************************************************
!*******************************************************************
 subroutine pbefunc(den,grad,fxc,dfxcdn,dfxcdg)

 use defs_basis
 use defs_excor
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp), intent(IN) :: den,grad    !density, magnitude of grad(density)
 real(dp), intent(OUT) :: fxc,dfxcdn,dfxcdg 

!Local variables ---------------------------------------
 real(dp) :: n,g,kf,rs,ks,s,t ! Variables depending on den & grad
 real(dp) :: ex,ec,Fx,H,A,pprs,ppt,At2,dFds,dHdt,decdrs,dHdrs,dHdA,dAdrs
 real(dp) :: term
 real(dp) :: Aofec,ddexp,Logofterm

!-------------------------------------------------------

 n=den
 if (n < machine_zero)  then
  fxc=0.d0; dfxcdn=0.d0; dfxcdg=0.d0
  return
 endif
 g=grad
 If (g < machine_zero) g=machine_zero

 kf=(3.d0*(pi**2)*n)**0.3333333333333333333333333333d0
 rs=(3.d0/(4.d0*pi*n))**0.3333333333333333333333333333d0
 ks=sqrt(4.d0*kf/pi)
 s=g/(2.d0*kf*n)
 t=g/(2.d0*ks*n)

 ex=-3.d0*kf/(4.d0*pi)
 pprs=sqrt(rs)*(b1+b3*rs)+rs*(b2+b4*rs)
 term=Logofterm(1.d0/(2.d0*AA*pprs))
 ec=-2.d0*AA*(1.d0+a1*rs)*term
 Fx=1.d0+kappa -kappa/(1.d0+(mu/kappa)*s*s)
 A=Aofec(ec)
 At2=A*t*t
 ppt=(1.d0+At2*(1.d0+At2))
 H=gamm*Logofterm((betabygamm)*(t*t)*((1.d0+At2)/ppt))

 fxc=n*(ex*Fx+ec+H)

 dFds = (2.d0*mu*s)/(1.d0+(mu/kappa)*(s**2))**2
 dHdt = (2.d0*t*beta*gamm*(1.d0+2.d0*At2))/((gamm*ppt+beta*t*t*(1.d0+At2))*ppt)
 decdrs=-(2.d0*AA*a1)*term &
   +((1.d0+a1*rs)*((b1+3*b3*rs)/(2.d0*sqrt(rs))+b2+2*b4*rs))/(pprs*(pprs+1.d0/(2.d0*AA)))
 dHdA=((2.d0+At2)*(At2*t*t*t*t*beta*gamm))/((gamm*ppt+beta*t*t*(1.d0+At2))*ppt)
 dAdrs=-ddexp(-ec/gamm)*A*A*decdrs/beta
 dHdrs=dHdA*dAdrs

 dfxcdn = (4.d0/3.d0)*ex*(Fx-dFds*s)+ec-(decdrs*rs)/3.d0+H-(dHdrs*rs)/3.d0 &
        - (7.d0/6.d0)*dHdt*t
 dfxcdg = n*ex*dFds/(2.d0*kf*n) + n*dHdt/(2.d0*ks*n)

 if ((abs(fxc).gt.1.d65).or.(abs(dfxcdn).gt.1.d65).or.(abs(dfxcdg).gt.1.d65)) then
  write(6,*) 'Problem in PBE',n,g,rs,s,t,ec,A,H
 endif

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine excor:
! input density in units of electrons/bohr**3
! output vxc potential in Hartree
! output exc total exchange-correlation energy/den
! uses Kohn-Sham exchange
! adds relativistic exchange iff relat=.true.
!    Rajagopal, J.Phys.C 11, L943 (1978),
!    MacDonald & Vosko, J.Phys.C 12, 2977 (1979)
! if itype eq 14 use Perdew & Wang, PRB, June 15, 1992
! if itype lt 13 use Vosko, Wilk, Nusair correlation,Can.J.Phys.58,1200(1980)
! if itype eq 13 use Wigner exchange correlation, Pickett/Bylander, Kleinman
!     B-K : Phys.Rev.B  vol.27 no.6 15 Mar 83
!     Pickett : Comp. Phys. Reports 9 (1989) p.115-198
!*******************************************************************
!*******************************************************************
 subroutine excor(deni,vxc,exc,relat,itype)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 integer :: itype
 real(dp) :: deni,vxc,exc
 logical :: relat

!Parameters ---------------------------------------
!Relativistic:
 real(dp), parameter :: balpha=137.03602d0
!VWN:
 real(dp), parameter :: x0=-0.10498d0,a=0.0621814d0,b=3.72744d0,c=12.9352d0
 real(dp), parameter :: xx0=x0*x0+b*x0+c,q=6.152029831d0
 real(dp), parameter :: c1=2.d0*b/q,c2=b*x0/xx0,c3=2.d0*(b+2.d0*x0)/q
!Perdew-Wang
 real(dp), parameter :: pa=0.031091d0,palp1=0.21370d0,pb1=7.5957d0, &
                        pb2=3.5876d0,pb3=1.6382d0,pb4=0.49294d0,tpa=2.d0*pa

!Local variables ---------------------------------------
 integer :: init=0
 real(dp) :: arg,beta,bgam,den,ec,exnr,exr,fa,ff,ffp,rfac,rrfac,rs,rsh,&
 &           vc,vxnr,vxr,x,xln,xx
 real(dp) :: ddlog

!-------------------------------------------------------

 if (init.ne.itype) then
  if (itype.eq.13) write(6,*) 'Wigner correlation'
  if (itype.gt.13) write(6,*) 'Perdew-Wang correlation'
  if (itype.lt.13) write(6,*) 'Ceperley-WVN correlation'
  init=itype
 endif

 den=max(deni,1.d-68)   ! Inelegant fix for small densities
 fa=(3.d0*pi*pi*den)**0.333333333333333333d0
 vxnr=-2.d0*fa/pi

 if (itype.ge.13) then
  rs=(3.d0/(4.d0*pi*den))**0.33333333333333333d0

  if (itype.eq.13) then
!  -- Wigner correlation --
   ec=-0.88d0/(7.79d0+rs)
   vc=-0.88d0*(4.d0*rs/3.d0+7.79d0)/((7.79d0+rs)**2)
  else
!  -- Perdew Wang correlation --
   rsh=sqrt(rs)
   ff=pb1*rsh+rs*(pb2+pb3*rsh+pb4*rs)
   ffp=0.5d0*pb1*rsh+rs*(pb2+1.5d0*pb3*rsh+2.d0*pb4*rs)
   xln=ddlog(1.d0+1.d0/(tpa*ff))
   ec=-tpa*(1.d0+palp1*rs)*xln
   vc=-tpa*(1.d0+0.6666666666666d0*palp1*rs)*xln &
      -tpa*(1.d0+palp1*rs)*ffp/(3.d0*ff*(1.d0+tpa*ff))
   ec=2.d0*ec
   vc=2.d0*vc
  endif
 else

  x=(0.75d0/(den*pi))**0.1666666666666666667d0
  xx=x*x+b*x+c
  arg=q/(2.d0*x+b)
  ec=a*(ddlog((x*x)/xx)+c1*atan(arg) &
    -c2*(ddlog(((x-x0)*(x-x0))/xx)+c3*atan(arg)))
  vc=ec-0.333333333333d0*a*(c/xx-(b*x0*x)/((x-x0)*xx))
 endif

 vxc=vxnr+vc
 exnr=0.75d0*vxnr
 exc=exnr+ec
      
 if(relat) then
  beta=fa/balpha
  bgam=sqrt(1.d0+beta*beta)
  if(beta.gt.1.d-3) then
   rfac=(1.d0-ddlog(beta+bgam)/(beta*bgam))
   rrfac=rfac*(bgam/beta)
  else
   rfac=0.666666666666666666666666d0*(beta*beta)
   rrfac=0.666666666666666666666666d0*beta
  endif
  vxr=-1.5d0*vxnr*rfac
  exr=-1.5d0*exnr*(rrfac*rrfac)
  vxc=vxc+vxr
  exc=exc+exr
 endif

 vxc=vxc*0.5d0   ! Rydberg to Hartree
 exc=exc*0.5d0

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine excorp:
! input deni=density
! input gd=abs(grad(density))/density
! input ggd=laplacian(density)/density
! input gggd=grad(density)*grad(abs(grad(density)))/density**2
! input density in units of electrons/bohr**3
! output vxc potential in Hartree
! output exc total exchange-correlation energy/den
! uses Perdew s gga-II program with no relativistic corrections
!  Perdew s pgm obtained from John Perdew 10-92
!  ref: PRB 46,6671 (1992)
!*******************************************************************
!*******************************************************************
 subroutine excorp(deni,gd,ggd,gggd,vxc,exc,relat,itype)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 integer :: itype
 real(dp) :: alfc,deni,gd,ggd,gggd,vxc,exc
 logical :: relat

!Commons -----------------------------------------------
 common/gas/ fk,sk,g,ec,ecrs,eczet
 real(dp) :: fk,sk,g,ec,ecrs,eczet
 common/mole/ ex,ecc,h,vx,vcup,vcdn,dvcup,dvcdn
 real(dp) ::  ex,ecc,h,vx,vcup,vcdn,dvcup,dvcdn

!Local variables ---------------------------------------
 real(dp), parameter :: small=1.d-12

 integer :: init=0
 real(dp) :: den,fa,fks,rs,s,t,tfa,tfks,u,uu,v,vv,ww,zet

!-------------------------------------------------------

 if (init.ne.itype) then
  if (itype.eq.13) write(6,*) 'Wigner correlation'
  if (itype.eq.14) write(6,*) 'Perdew-Wang correlation'
  if (itype.lt.13) write(6,*) 'Ceperley-WVN correlation'
  if (itype.eq.15) write(6,*) 'Perdew''s gga-II ex & corr'
  if (itype.ne.15) then
   write(6,*) 'exiting -- itype should be 15'
   stop
  endif
  init=itype
 endif

 den=max(deni,1.d-68)  ! Inelegant fix for small densities
 if (den.le.small) then
  vxc=0.d0;exc=0.d0
  return
 endif

 fa=(3.d0*pi*pi*den)**0.333333333333333333d0
 fk=fa
 tfa=2.d0*fa
 s=gd/tfa
 u=gggd/tfa**3
 v=ggd/tfa**2
 ex=0.d0
 ec=0.d0
 h=0.d0
 vcup=0.d0
 vcdn=0.d0
 dvcup=0.d0
 dvcdn=0.d0
 call exch(den,s,u,v,ex,vx)
 if(den.le.1.d-18) go to 1111
 rs=(3.d0/(4.d0*pi*den))**0.33333333333333333d0
 zet=0.d0
 call corlsd(rs,zet,ec,vcup,vcdn,ecrs,eczet,alfc)
 fks=sqrt(4.d0*fa/pi)
 sk=fks
 g=1.d0
 tfks=2.d0*fks
 t=gd/tfks
 uu=gggd/tfks**3
 vv=ggd/tfks**2
 ww=0.d0
 call corgga(rs,zet,t,uu,vv,ww,h,dvcup,dvcdn)
1111 continue
 exc=2.d0*(ex+ec+h)
 vxc=2.d0*vx+vcup+vcdn+dvcup+dvcdn
 ecc=ec

 vxc=vxc*0.5d0   ! Rydberg to Hartree
 exc=exc*0.5d0

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine exch:
! Generate the generalized gradient approximation for
! the exchange-correlation energy and potential (j.p. perdew
! and y. wang, 1991).  they are keyed to the programmable
! expressions in the notes of 1/23/91.
! main program tests the expression for the functional derivative
! (potential) by application to unequal exponential up- and
! down-spin densities.
! * energies in hartrees, distances in bohrs
! * gga91 exchange for a spin-unpolarized electronic system
!  input d : density
!  input s:  abs(grad d)/(2*kf*d)
!  input u:  (grad d)*grad(abs(grad d))/(d**2 * (2*kf)**3)
!  input v: (laplacian d)/(d*(2*kf)**2)
!  output:  exchange energy per electron (ex) and potential (vx)
!*******************************************************************
!*******************************************************************
 subroutine exch(d,s,u,v,ex,vx)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: d,s,u,v,ex,vx

!Parameters --------------------------------------------
 real(dp) :: a1=0.19645d0,a2=0.27430d0,a3=0.15084d0,a4=100.d0
 real(dp) :: ax=-0.7385588d0,a=7.7956d0,b1=0.004d0
 real(dp) :: thrd=0.333333333333d0,thrd4=1.33333333333d0

!Local variables ---------------------------------------
 real(dp) :: f,fs,fss,fac,p0,p1,p2,p3,p4,p5,p6,p7,p8,p9,&
&            p10,p11,s2,s3,s4
 real(dp) :: ddexp,ddlog

!-------------------------------------------------------

 fac = ax*d**thrd
 s2 = s*s
 s3 = s2*s
 s4 = s2*s2
 p0 = 1.d0/sqrt(1.d0+a*a*s2)
 p1 = ddlog(a*s+1.d0/p0)

!...nawh10/92      p2 = dexp(-a4*s2)
 p2 = ddexp(-a4*s2)
 p3 = 1.d0/(1.d0+a1*s*p1+b1*s4)
 p4 = 1.d0+a1*s*p1+(a2-a3*p2)*s2
 f = p3*p4
 ex = fac*f

!Energy done. now the potential:
 p5 = b1*s2-(a2-a3*p2)
 p6 = a1*s*(p1+a*s*p0)
 p7 = 2.d0*(a2-a3*p2)+2.d0*a3*a4*s2*p2-4.d0*b1*s2*f
 fs = p3*(p3*p5*p6+p7)
 p8 = 2.d0*s*(b1-a3*a4*p2)
 p9 = a1*p1+a*a1*s*p0*(3.d0-a*a*s2*p0*p0)
 p10 = 4.d0*a3*a4*s*p2*(2.d0-a4*s2)-8.d0*b1*s*f-4.d0*b1*s3*fs
 p11 = -p3*p3*(a1*p1+a*a1*s*p0+4.d0*b1*s3)
 fss = p3*p3*(p5*p9+p6*p8)+2.d0*p3*p5*p6*p11+p3*p10+p7*p11
 vx = fac*(thrd4*f-(u-thrd4*s3)*fss-v*fs)

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine corlsd:
!  Uniform-gas correlation of perdew and wang 1991
!  input: seitz radius (rs), relative spin polarization (zet)
!  output: correlation energy per electron (ec), up- and down-spin
!     potentials (vcup,vcdn), derivatives of ec wrt rs (ecrs) & zet (eczet)
!  output: correlation contribution (alfc) to the spin stiffness
!*******************************************************************
!*******************************************************************
 subroutine corlsd(rs,zet,ec,vcup,vcdn,ecrs,eczet,alfc)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: rs,zet,ec,vcup,vcdn,ecrs,eczet,alfc

!Parameters --------------------------------------------
 real(dp) :: gam=0.5198421d0,fzz=1.709921d0
 real(dp) :: thrd=0.333333333333d0,thrd4=1.33333333333d0

!Local variables ---------------------------------------
 real(dp) :: alfm,alfrsm,comm,ep,eprs,eu,eurs,f,fz,z4

!-------------------------------------------------------

 f = ((1.d0+zet)**thrd4+(1.d0-zet)**thrd4-2.d0)/gam
 call gcor(0.0310907d0,0.21370d0,7.5957d0,3.5876d0,1.6382d0, &
           0.49294d0,1.00d0,rs,eu,eurs)
 call gcor(0.01554535d0,0.20548d0,14.1189d0,6.1977d0,3.3662d0, &
           0.62517d0,1.00d0,rs,ep,eprs)
 call gcor(0.0168869d0,0.11125d0,10.357d0,3.6231d0,0.88026d0, &
           0.49671d0,1.00d0,rs,alfm,alfrsm)
!alfm is minus the spin stiffness alfc
 alfc = -alfm
 z4 = zet**4
 ec = eu*(1.d0-f*z4)+ep*f*z4-alfm*f*(1.d0-z4)/fzz

!Energy done. now the potential:
 ecrs = eurs*(1.d0-f*z4)+eprs*f*z4-alfrsm*f*(1.d0-z4)/fzz
 fz = thrd4*((1.d0+zet)**thrd-(1.d0-zet)**thrd)/gam
 eczet = 4.d0*(zet**3)*f*(ep-eu+alfm/fzz)+fz*(z4*ep-z4*eu &
       -(1.d0-z4)*alfm/fzz)
 comm = ec -rs*ecrs/3.d0-zet*eczet
 vcup = comm + eczet
 vcdn = comm - eczet

 end subroutine




!*******************************************************************
!*******************************************************************
! Subroutine corgga:
!  gga91 correlation
!  input rs: seitz radius
!  input zet: relative spin polarization
!  input t: abs(grad d)/(d*2.*ks*g)
!  input uu: (grad d)*grad(abs(grad d))/(d**2 * (2*ks*g)**3)
!  input vv: (laplacian d)/(d * (2*ks*g)**2)
!  input ww:  (grad d)*(grad zet)/(d * (2*ks*g)**2
!  output h: nonlocal part of correlation energy per electron
!  output dvcup,dvcdn:  nonlocal parts of correlation potentials
!*******************************************************************
!*******************************************************************
 subroutine corgga(rs,zet,t,uu,vv,ww,h,dvcup,dvcdn)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: rs,zet,t,uu,vv,ww,h,dvcup,dvcdn

!Commons -----------------------------------------------
 common/gas/ fk,sk,g,ec,ecrs,eczet
 real(dp) :: fk,sk,g,ec,ecrs,eczet

!Parameters --------------------------------------------
 real(dp) :: xnu=15.75592d0,cc0=0.004235d0,cx=-0.001667212d0,alf=0.09d0
 real(dp) :: c1=0.002568d0,c2=0.023266d0,c3=7.389d-6,c4=8.723d0
 real(dp) :: c5=0.472d0,c6=7.389d-2,a4=100.d0
 real(dp) :: thrdm=-0.333333333333d0,thrd2=0.666666666667d0

!Local variables ---------------------------------------
 real(dp) :: b,b2,bec,bet,bg,cc,ccrs,coeff,comm,delt
 real(dp) :: fact0,fac,fact1,fact2,fact3,fact4,fact5
 real(dp) :: g3,g4,gz,h0
 real(dp) :: h0b,h0bt,h0rs,h0rst,h0t,h0tt,h0z,h0zt
 real(dp) :: h1,h1rs,h1rst,h1t,h1tt,h1z,h1zt
 real(dp) :: hrs,hrst,ht,htt,hz,hzt
 real(dp) :: pon,pref,q4,q5,q6,q7,q8,q9,r0,r1,r2,r3,r4,rs2,rs3,rsthrd,t2,t4,t6
 real(dp) :: ddexp,ddlog

!-------------------------------------------------------

 bet = xnu*cc0
 delt = 2.d0*alf/bet
 g3 = g**3
 g4 = g3*g
!... nawh10/92      pon = -delt*ec/(g3*bet)
!... nawh10/92      b = delt/(dexp(pon)-1.d0)
 pon =  delt*ec/(g3*bet)
 b = delt/(ddexp(-pon)-1.d0)
 b2 = b*b
 t2 = t*t
 t4 = t2*t2
 t6 = t4*t2
 rs2 = rs*rs
 rs3 = rs2*rs
 q4 = 1.d0+b*t2
 q5 = 1.d0+b*t2+b2*t4
 q6 = c1+c2*rs+c3*rs2
 q7 = 1.d0+c4*rs+c5*rs2+c6*rs3
 cc = -cx + q6/q7
 r0 = (sk/fk)**2
 r1 = a4*r0*g4
 coeff = cc-cc0-3.d0*cx/7.d0
 r2 = xnu*coeff*g3
!...  nawh10/92      r3 = dexp(-r1*t2)
 r3 = ddexp(-r1*t2)
!...  nawh10/92      h0 = g3*(bet/delt)*ddlog(1.d0+delt*q4*t2/q5)
 h0 = -ec + g3*(bet/delt)*ddlog(1.d0+(ddexp(pon)-1.d0)/q5)
 h1 = r3*r2*t2
 h = h0+h1

!Energy done. now the potential:
 ccrs = (c2+2.*c3*rs)/q7 - q6*(c4+2.*c5*rs+3.*c6*rs2)/q7**2
 rsthrd = rs/3.d0
 r4 = rsthrd*ccrs/coeff
 gz = ((1.d0+zet)**thrdm - (1.d0-zet)**thrdm)/3.d0
 fac = delt/b+1.d0
 bg = -3.d0*b2*ec*fac/(bet*g4)
 bec = b2*fac/(bet*g3)
 q8 = q5*q5+delt*q4*q5*t2
 q9 = 1.d0+2.d0*b*t2
 h0b = -bet*g3*b*t6*(2.d0+b*t2)/q8
 h0rs = -rsthrd*h0b*bec*ecrs
 fact0 = 2.d0*delt-6.d0*b
 fact1 = q5*q9+q4*q9*q9
 h0bt = 2.d0*bet*g3*t4*((q4*q5*fact0-delt*fact1)/q8)/q8
 h0rst = rsthrd*t2*h0bt*bec*ecrs
 h0z = 3.d0*gz*h0/g + h0b*(bg*gz+bec*eczet)
 h0t = 2.*bet*g3*q9/q8
 h0zt = 3.d0*gz*h0t/g+h0bt*(bg*gz+bec*eczet)
 fact2 = q4*q5+b*t2*(q4*q9+q5)
 fact3 = 2.d0*b*q5*q9+delt*fact2
 h0tt = 4.d0*bet*g3*t*(2.d0*b/q8-(q9*fact3/q8)/q8)
 h1rs = r3*r2*t2*(-r4+r1*t2/3.d0)
 fact4 = 2.d0-r1*t2
 h1rst = r3*r2*t2*(2.d0*r4*(1.d0-r1*t2)-thrd2*r1*t2*fact4)
 h1z = gz*r3*r2*t2*(3.d0-4.d0*r1*t2)/g
 h1t = 2.d0*r3*r2*(1.d0-r1*t2)
 h1zt = 2.d0*gz*r3*r2*(3.d0-11.d0*r1*t2+4.d0*r1*r1*t4)/g
 h1tt = 4.d0*r3*r2*r1*t*(-2.d0+r1*t2)
 hrs = h0rs+h1rs
 hrst = h0rst+h1rst
 ht = h0t+h1t
 htt = h0tt+h1tt
 hz = h0z+h1z
 hzt = h0zt+h1zt
 comm = h+hrs+hrst+t2*ht/6.d0+7.d0*t2*t*htt/6.d0
 pref = hz-gz*t2*ht/g
 fact5 = gz*(2.d0*ht+t*htt)/g
 comm = comm-pref*zet-uu*htt-vv*ht-ww*(hzt-fact5)
 dvcup = comm + pref
 dvcdn = comm - pref

 end subroutine

!*******************************************************************
!*******************************************************************
! Subroutine gcor:
!  called by subroutine corlsd
!*******************************************************************
!*******************************************************************
 subroutine gcor(a,a1,b1,b2,b3,b4,p,rs,gg,ggrs)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: a,a1,b1,b2,b3,b4,p,rs,gg,ggrs

!Local variables ---------------------------------------
 real(dp) :: p1,q0,q1,q2,q3,rs12,rs32,rsp
 real(dp) :: ddlog

!-------------------------------------------------------

 p1 = p + 1.d0
 q0 = -2.d0*a*(1.d0+a1*rs)
 rs12 = sqrt(rs)
 rs32 = rs12**3
 rsp = rs**p
 q1 = 2.d0*a*(b1*rs12+b2*rs+b3*rs32+b4*rs*rsp)
 q2 = ddlog(1.d0+1.d0/q1)
 gg = q0*q2
 q3 = a*(b1/rs12+2.d0*b2+3.d0*b3*rs12+2.d0*b4*p1*rsp)
 ggrs = -2.d0*a*a1*q2-q0*q3/(q1**2+q1)

 end subroutine




!*******************************************************************
!*******************************************************************
! Function Logofterm:
!  Evaluates log(1+term)
! -- needed to take care of behavior for small term 
!*******************************************************************
!*******************************************************************
 function Logofterm(term)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: term, Logofterm

!Local variables ---------------------------------------
 real(dp) :: ddlog
 
!-------------------------------------------------------

 if (abs(term)>machine_precision) then
  Logofterm=ddlog(1.d0+term)
 else
  Logofterm=term
 endif

 end function




!*******************************************************************
!*******************************************************************
! Function Aofec:
! -- needed to take care of behavior for small ec
!*******************************************************************
!*******************************************************************
 function Aofec(ec)

 use defs_basis
 use defs_excor
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: ec, Aofec

!Local variables ---------------------------------------
 real(dp) :: ddexp

!-------------------------------------------------------

 if (abs(ec)>machine_precision) then
  Aofec=betabygamm/(ddexp(-ec/gamm)-1.d0)
 elseif (abs(ec)>machine_zero) then
  Aofec=beta/(-ec)
 else
  Aofec=-beta*dsign(machine_infinity,ec)
 endif

 end function




!*******************************************************************
!*******************************************************************
! Function ddlog:
!*******************************************************************
!*******************************************************************
 function ddlog(arg)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: arg,ddlog

!-------------------------------------------------------

 if (arg>maxlogarg) then
  ddlog=maxlog
 else if (arg<minlogarg) then
  ddlog=minlog
 else
  ddlog=log(arg)
 endif

 end function



 
!*******************************************************************
!*******************************************************************
! Function ddexp:
!*******************************************************************
!*******************************************************************
 function ddexp(arg)

 use defs_basis
 use defs_mach

 implicit none

!Arguments ---------------------------------------------
 real(dp) :: arg,ddexp

!-------------------------------------------------------

 if (arg>maxexparg) then
  ddexp=maxexp
 else if (arg<minexparg) then
  ddexp=minexp
 else
  ddexp=exp(arg)
 endif

 end function




!*******************************************************************
!*******************************************************************
! Subroutine Init_defs_mach:
!*******************************************************************
!*******************************************************************
 Subroutine Init_defs_mach

 use defs_basis
 use defs_mach

 implicit none

!Local variables ---------------------------------------
 integer :: i
 real(dp) :: tmp, a1,a2,a3

!-------------------------------------------------------

!Calculate machine accuracy
 machine_precision = 0
 a1 = 4.d0/3.d0
 do while (machine_precision == 0.d0)
  a2 = a1 - 1.d0
  a3 = a2 + a2 + a2
  machine_precision = ABS(a3 - 1.d0)
 enddo

 machine_zero= machine_precision**4
 machine_infinity = 1.d0/machine_zero

 minlogarg=machine_precision; minlog=log(minlogarg)
 maxlogarg=1.d0/machine_precision; maxlog=log(maxlogarg)
 minexparg=log(machine_precision);  minexp=0.d0
 maxexparg=-log(machine_precision);  maxexp=exp(maxexparg)
   
 end subroutine
