!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdpawps1
!! NAME
!! rdpawps1
!! 
!! FUNCTION
!! Read the header of a PAW pseudopotential file generated by AtomPAW
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!! 
!! INPUTS
!!  filename(3)= input file names (atomicdata, density, potential)
!!  unit(3)= input unit numbers
!!   
!! OUTPUT
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %bigmesh_size= Dimension of radial mesh for read coredens/valedens
!!    %core_charge= Core charge
!!    %lambda= Lambda in gaussian type g(r)
!!    %lmn_size= Number of elements for the paw basis
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %proj_meshsz= Dimension of radial mesh for tproj
!!    %pspxc_atom= AtomPaw s code number for the exchange-correlation
!!    %rad_step= Step corresponding to radial mesh
!!    %shape_type= Shape function type
!!    %sigma= Sigma for gaussian type g(r)
!!    %title= Title for pseudopotential
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!
!! SOURCE

 subroutine rdpawps1(filename,pshead,unit)

 use defs_basis
 use defs_pawps
 
 implicit none

!Arguments ---------------------------------------------
 integer :: unit(3)
 character*(fnlen) :: filename(3)
!This type is defined in defs_pawps
 type(pshead_type) :: pshead

!Local variables ---------------------------------------
 integer :: ib
 logical :: endfile
 character*(fnlen) :: line,readline,strg

! *************************************************************************

!Open the atomicdata file for reading
 open(unit=unit(1),file=filename(1),form='formatted',status='old')

!Start a reading loop
 endfile=.false.
 do while (.not.endfile)
   read(unit(1),'(a)',err=10,end=10) readline
   line=adjustl(readline);goto 20
10 line="";endfile=.true.
20 continue

!--Read TITLE
  if (line(1:8)=='ATOMTYPE') then
   read(unit=line(9:fnlen),fmt=*) strg
   write(unit=pshead%title,fmt='(3a)') &
&   "Paw atomic data for element ",trim(ADJUSTL(strg)),&
&   " - Generated by AtomPAW (N. Holzwarth)"
  endif

!--Read BASIS SIZE
  if (line(1:10)=='BASIS_SIZE') then
   read(unit=line(11:fnlen),fmt=*) pshead%basis_size
   if (pshead%basis_size>20) then
    write(6,'(a)') "Error in rdpawps1: basis size too large (>20) !"
    close(unit(1));stop
   endif
  endif

!--Read ORBITALS
  if (line(1:8)=='ORBITALS') then
   read(unit(1),'(a)',err=11,end=11) readline
   line=adjustl(readline);goto 21
11 write(6,'(a)') "Error in rdpawps1: error reading orbitals !"
   close(unit(1));stop
21 read(unit=line(1:fnlen),fmt=*) &
&      (pshead%orbitals(ib),ib=1,pshead%basis_size)
  endif

!--Read MESH SIZE
  if (line(1:9)=='MESH_SIZE') then
   read(unit=line(10:fnlen),fmt=*) pshead%mesh_size
   pshead%proj_meshsz=4.d0*pshead%mesh_size
  endif

!--Read MESH STEP
  if (line(1:9)=='MESH_STEP') &
&    read(unit=line(10:fnlen),fmt=*) pshead%rad_step

!--Read ATOMIC CHARGE
  if (line(1:13)=='ATOMIC_CHARGE') &
&    read(unit=line(14:fnlen),fmt=*) pshead%atomic_charge

!--Read CORE CHARGE
  if (line(1:11)=='CORE_CHARGE') &
&    read(unit=line(12:fnlen),fmt=*) pshead%core_charge

!--Read EXCHANGE-CORRELATION TYPE
  if (line(1:10)=='ATOMXCTYPE') then
   read(unit=line(11:fnlen),fmt=*) strg
   if (trim(ADJUSTL(strg))=="LDA-PW") then
    pshead%pspxc_atom=14
   elseif (trim(ADJUSTL(strg))=="GGA-PBE") then
    pshead%pspxc_atom=16
   else
    write(6,'(a)') "Error in rdpawps1: unknown XC type !"
    close(unit(1));stop
   endif
  endif

!--Read SHAPE TYPE
  if (line(1:10)=='SHAPE_TYPE') then
   read(unit=line(11:fnlen),fmt=*) strg
   if (trim(ADJUSTL(strg))=="exshape") then
    pshead%shape_type=1
   elseif (trim(ADJUSTL(strg))=="sinc2") then
    pshead%shape_type=2
    pshead%lambda=0;pshead%sigma=0.d0
   else
    write(6,'(a)') "Error in rdpawps1: unknown shape type !"
    close(unit(1));stop
   endif
  endif

!End of reading loop
 enddo

!Close the atomicdata file
 close(unit(1))

!Open the density file for reading
 open(unit=unit(2),file=filename(2),form='formatted',status='old')

!Read file in order to count number of lines
 endfile=.false.;pshead%bigmesh_size=0
 do while (.not.endfile)
   read(unit(2),'(a)',err=12,end=12)
   pshead%bigmesh_size=pshead%bigmesh_size+1
   goto 22
12 endfile=.true.
22 continue
 enddo
 pshead%bigmesh_size=pshead%bigmesh_size+1

!Close the density file
 close(unit(2))

!Verify: bigmesh_size>mesh_size
 if (pshead%bigmesh_size<pshead%mesh_size) then
  write(6,'(a)') "Error in rdpawps1:"
  write(6,'(a)') "   Mesh size in density file (bigmesh_size)"
  write(6,'(a)') "   must be greater than mesh size in atomicdata"
  write(6,'(a)') "   file (mesh_size) !"
  stop
 endif


!Here must compute lmn_size
!--------------------------
 pshead%lmn_size=0
 do ib=1,pshead%basis_size
  pshead%lmn_size=pshead%lmn_size+2*pshead%orbitals(ib)+1
 enddo

 end subroutine
!!***
