!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdpawps2
!! NAME
!! rdpawps2
!! 
!! FUNCTION
!! Read the body of a PAW pseudopotential file generated by AtomPAW
!! (the header must have been read before)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!! 
!! INPUTS
!!  filename(3)= input file names (atomicdata, density, potential)
!!  unit(3)= input unit numbers
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %bigmesh_size= Dimension of radial mesh for read coredens/valedens
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %proj_meshsz= Dimension of radial mesh for tproj
!!    %shape_type= Shape function type
!!  pawdata
!!    %rad(bigmesh_size)= Coordinates of the radial grid
!!   
!! OUTPUT
!!  pawarray
!!    %coreden4pr2(bigmesh_size)= Core density multiplied by 4Pi.r2
!!    %rveff(bigmesh_size)= Effective all electron atomic potential*r
!!    %tvaleden4pr2(bigmesh_size) = Density 4*pi*r^2*tn1,
!!                                  coming from the code Atompaw 
!!    %valeden4pr2(bigmesh_size) = Density 4*pi*r^2*n1,
!!                                 coming from the code Atompaw 
!!  pawps
!!    %phi(mesh_size,basis_size)= PAW atomic wavefunctions
!!                                on the radial grid
!!    %tphi(mesh_size,basis_size)= PAW atomic pseudo-wavefunctions
!!                                 on the radial grid
!!    %tproj(proj_meshsz,basis_size)= PAW projectors
!!                                    on the radial grid
!!
!! SIDE EFFECTS
!!  pshead
!!    %lambda= Lambda in gaussian type g(r)
!!    %sigma= Sigma for gaussian type g(r)
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!
!! SOURCE

 subroutine rdpawps2(filename,pawarray,pawdata,pawps,pshead,unit)

 use defs_basis
 use defs_pawps
 
 implicit none
 
!Arguments ---------------------------------------------
 integer :: unit(3)
 character*(fnlen) :: filename(3)
!These types are defined in defs_pawps
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer :: ib,ir1,ir2,ir
 logical :: found,endfile,shapeffound
 logical,allocatable :: phifound(:),tphifound(:),&
&                       tprojfound(:)
 real(dp) :: sf1,sf2,r1,r2
 real(dp), allocatable :: readrad(:),shapef(:)
 character*(fnlen) :: line,readline

! *********************************************************************


! ==================================================
! ====== 1-READ  ATOMICDATA FILE FROM AtomPAW ======
! ==================================================

!Init some flags
 allocate(phifound(pshead%basis_size));phifound=.false.
 allocate(tphifound(pshead%basis_size));tphifound=.false.
 allocate(tprojfound(pshead%basis_size));tprojfound=.false.
 shapeffound=.true.;if (pshead%shape_type==1) shapeffound=.false.

!Open the atomicdata file for reading
 open(unit=unit(1),file=filename(1),form='formatted',status='old')

!Start a reading loop
 endfile=.false.
 do while (.not.endfile)
   read(unit(1),'(a)',err=10,end=10) readline
   line=adjustl(readline);goto 20
10 line="";endfile=.true.
20 continue

!--Read WAVE FUNCTIONS PHI
  if (line(1:4)=='PHI ') then
   read(unit=line(5:fnlen),fmt=*) ib
   if (ib>pshead%basis_size) then
    write(6,'(a)') "Error in rdpawps2: PHI(ib) for ib>basis_sz !"
    close(unit(1));stop
   endif
   read(unit(1),*) (pawps%phi(ir,ib),ir=1,pshead%mesh_size)
   phifound(ib)=.true.
  endif

!--Read PSEUDO WAVE FUNCTIONS TPHI
  if (line(1:5)=='TPHI ') then
   read(unit=line(6:fnlen),fmt=*) ib
   if (ib>pshead%basis_size) then
    write(6,'(a)') "Error in rdpawps2: TPHI(ib) for ib>basis_sz !"
    close(unit(1));stop
   endif
   read(unit(1),*) (pawps%tphi(ir,ib),ir=1,pshead%mesh_size)
   tphifound(ib)=.true.
  endif

!--Read PROJECTORS TPROJ
  if (line(1:10)=='TPROJECTOR') then
   read(unit=line(11:fnlen),fmt=*) ib
   if (ib>pshead%basis_size) then
    write(6,'(a)') "Error in rdpawps2: TPROJ(ib) for ib>basis_sz !"
    close(unit(1));stop
   endif
   read(unit(1),*) (pawps%tproj(ir,ib),ir=1,pshead%mesh_size)
   tprojfound(ib)=.true.
  endif

!--Read EXPONENTIAL SHAPE FUNCTION
!  and  DEDUCE GAUSSIAN PARAMETERS
  if (pshead%shape_type==1.and.line(1:10)=='SHAPE_FUNC') then
   allocate(shapef(pshead%mesh_size))
   read(unit(1),*) (shapef(ir),ir=1,pshead%mesh_size)
   shapeffound=.true.
   ir1=pshead%mesh_size/2;ir2=3*pshead%mesh_size/4
   if (ir1==ir2) ir2=pshead%mesh_size
   r1=pawdata%rad(ir1);r2=pawdata%rad(ir2)
   sf1=shapef(ir1);sf2=shapef(ir2)
   pshead%lambda=nint(log(log(sf1)/log(sf2))/log(r1/r2))
   pshead%sigma=r1/(-log(sf1))**(1.d0/dble(pshead%lambda))
   deallocate(shapef)
  endif

!End of reading loop
 enddo

!Close the atomicdata file
 close(unit(1))

!Test flags: is anything OK ? 
 do ib=1,pshead%basis_size
  if (.not.phifound(ib)) &
&  write(6,'(a,i2,a)') "Error in rdpawps2: PHI(",ib,") not found !"
  if (.not.tphifound(ib)) &
&  write(6,'(a,i2,a)') "Error in rdpawps2: TPHI(",ib,") not found !"
  if (.not.tprojfound(ib)) &
&  write(6,'(a,i2,a)') "Error in rdpawps2: TPROJ(",ib,") not found !"
 enddo
 if (.not.shapeffound) &
&  write(6,'(a,i2,a)') "Error in rdpawps2: SHAPE FUNC not found !"
 found=shapeffound
 do ib=1,pshead%basis_size
  found=found.and.phifound(ib).and.tphifound(ib).and.tprojfound(ib)
 enddo
 if (.not.found) stop
 deallocate(phifound,tphifound,tprojfound)



! ==================================================
! ====== 2-READ  DENSITY FILE FROM AtomPAW    ======
! ==================================================

!Open the density file for reading
 open(unit=unit(2),file=filename(2),form='formatted',status='old')

!Read coredens, valedens and tvaledens
 allocate(readrad(pshead%bigmesh_size));readrad(1)=zero
 pawarray%coreden4pr2(1)=zero
 pawarray%valeden4pr2(1)=zero;pawarray%tvaleden4pr2(1)=zero
 do ir=2,pshead%bigmesh_size
  read(unit(2),*) readrad(ir),pawarray%coreden4pr2(ir),&
&                 pawarray%valeden4pr2(ir),pawarray%tvaleden4pr2(ir)
 enddo

!Close the density file
 close(unit(2))

!Verify consistency of read radii in density file
 do ir=1,pshead%bigmesh_size
  if (abs(pawdata%rad(ir)-readrad(ir))>1.d-6) then
   write(6,'(a)') "Error in rdpawps2: radii read in density file"
   write(6,'(a)') "                   do not match with i*step !"
   stop
  endif
 enddo



! ==================================================
! ====== 3-READ  POTENTIAL FILE FROM AtomPAW  ======
! ==================================================

!Open the potential file for reading
 open(unit=unit(3),file=filename(3),form='formatted',status='old')

!Read r*veff
 pawarray%rveff(1)=zero
 do ir=2,pshead%bigmesh_size
  read(unit(3),*) readrad(ir),pawarray%rveff(ir)
 enddo

!Convert r*veff from Rydberg to Hartree
 pawarray%rveff=pawarray%rveff*0.5d0

!Close the potential file
 close(unit(3))

!Verify consistency of read radii in potential file
 do ir=1,pshead%bigmesh_size
  if (abs(pawdata%rad(ir)-readrad(ir))>1.d-6) then
   write(6,'(a)') "Error in rdpawps2: radii read in potential file"
   write(6,'(a)') "                   do not match with i*step !"
   stop
  endif
 enddo
 deallocate(readrad)

 end subroutine
!!***
