!{\src2tex{textfont=tt}}
!!****f* ABINIT/wrpawps
!! NAME
!! wrpawps
!! 
!! FUNCTION
!! Write a PAW pseudopotential file formatted for Abinit
!!
!! COPYRIGHT
!! Copyright (C) 1998-2004 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  filename= output file name for Abinit
!!  funit= output unit number
!!  loggrd
!!    %meshsz=mesh size for the logarithmic grid
!!    %uselog=TRUE if data are transfered on a log. grid before being written
!!    %log_step=logarithmic step for the logarithmic grid
!!    %rad_step=radial step for the logarithmic grid
!!  pawps
!!    %coredens(bigmesh_size)= Core density of the atom
!!    %tcoredens(bigmesh_size)= Pseudo-core density of the atom
!!    %dij0(lmn2_size)= Part of the Dij term calculated in the psp part
!!    %phi(mesh_size,basis_size)= PAW atomic wavefunctions
!!                                on the radial grid
!!    %tphi(mesh_size,basis_size)= PAW atomic pseudo-wavefunctions
!!                                 on the radial grid
!!    %tproj(proj_meshsz,basis_size)= PAW projectors
!!                                 on the radial grid
!!    %rhoij0= Atomic initialization of rhoij
!!    %vhtnzc(bigmesh_size)= Hartree potential of the ps-density
!!                           of the nucleus + core electrons
!!  pawrso
!!    %userso=TRUE if REAL Space Optimization is required
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %core_charge= Core charge
!!    %core_meshsz= Dimension of radial mesh for (t)coredens for coredens<pgiven prec
!!    %creatorid= ID of psp generator (here creatorID=1 !)
!!    %lambda= Lambda in gaussian type g(r)
!!    %lmax= Maximum value of l
!!    %lmn_size= Number of elements for the paw basis
!!    %mesh_size= Dimension of radial mesh for phi, tphi ...
!!    %mesh_type=  Flag defining ther radial grid type
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %proj_meshsz= Dimension of radial mesh for tproj
!!    %pspcod= Psp code number for Abinit (here PAW->pspcod=7 !)
!!    %pspxc_abinit= Abinit s code number for the exchange-correlation
!!    %rad_step= Step corresponding to radial mesh
!!    %rc_proj= Sphere radius for tproj
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type
!!    %sigma= Sigma for gaussian type g(r)
!!    %title= Title for pseudopotential
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhnzc
!!
!! NOTES
!!  File format of formatted PAW psp input for Abinit:
!!  --------------------------------------------------
!!  (1) title (character) line
!!  (2) znucl, zion, pspdat
!!  (3) pspcod, pspxc, lmax, lloc, mmax, r2well
!!  (4) psp_version, creatorID (here, creatorID=1)
!!  (5) basis_size, lmn_size
!!  (6) orbitals (for l=1 to basis_size)
!!  (7) number_of_meshes
!!  For imsh=1 to number_of_meshes
!!      (8)  mesh_index, mesh_type ,mesh_size, rad_step[, log_step]
!!  (9) r_cut(SPH)
!!  (10) shape_type[, lambda, sigma]
!!  For iln=1 to basis_size
!!      (11)  comment(character)
!!      (12) phi(r) (for ir=1 to mesh_size(SPH))
!!  For iln=1 to basis_size
!!      (13) comment(character)
!!      (14) tphi(r) (for ir=1 to mesh_size(SPH))
!!  For iln=1 to basis_size
!!      (15) comment(character)
!!      (16) tproj(r) (for ir=1 to mesh_size(PROJ))
!!  (17) comment(character)
!!  (18) core_density (for ir=1 to mesh_size(SPH))
!!  (19) comment(character)
!!  (20) tcore_density (for ir=1 to mesh_size(SPH))
!!  (21) comment(character)
!!  (22) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (23) comment(character)
!!  (24) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (25) comment(character)
!!  (26) VHntzc(r) (=Vloc) (for ir=1 to mesh_size(VLOC))
!!  --------------------------------------------------
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!      date_and_time,splinefit
!!
!! SOURCE

 subroutine wrpawps(fname,loggrd,pawps,pawrso,pshead,funit)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 integer :: funit
 character*(fnlen) :: fname
!These types are defined in defs_pawps
 type(loggrd_type)   :: loggrd
 type(pawps_type)   :: pawps
 type(pawrso_type)   :: pawrso
 type(pshead_type)  :: pshead

!Local variables ---------------------------------------
 integer :: coremeshsz,ib,icoremesh,ii,il,ilmn,iprojmesh,ir
 integer :: ivlocmesh,jlmn,nmesh,sphmeshsz,vlocmeshsz
 character*8 :: strdate
 integer :: meshsz(3),meshtp(3)
 real(dp) :: radstp(3),logstp(3)
 real(dp), allocatable :: ffit(:),rr_log(:)

! *************************************************************************

!Test consistency of SPH and PROJ meshes
 if ((.not.pawrso%userso).and.(pshead%mesh_size/=pshead%proj_meshsz)) then
  write(6,'(/,a)')   'Error (wrpawps):'
  write(6,'(a)')     '  Mesh size for tproj differs'
  write(6,'(a,i2,a)')'  from mesh size for (t)phi !'
  stop 'Program stopped before end'
 endif

!Meshes definitions (if necessary define a logarithmic radial grid)
 if (loggrd%uselog) then
  sphmeshsz=loggrd%meshsz
  loggrd%rad_step=(pshead%rc_sph*(1.d0-1.d-12))*exp(-loggrd%log_step*dble(loggrd%meshsz-2))
  coremeshsz=int((1.d0+1.d-12)*log(pshead%rad_step*dble(pshead%core_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
  vlocmeshsz=int((1.d0+1.d-12)*log(pshead%rad_step*dble(pshead%vloc_meshsz-1)/loggrd%rad_step)/loggrd%log_step)+2
  pshead%rc_sph=loggrd%rad_step*exp(loggrd%log_step*dble(sphmeshsz-2))
  allocate(rr_log(max(sphmeshsz,vlocmeshsz)));rr_log(1)=zero
  do ir=2,max(sphmeshsz,vlocmeshsz)
   rr_log(ir)=loggrd%rad_step*exp(loggrd%log_step*dble(ir-2))
  enddo
  write(6,'(3(/,a),2(/,a,g10.4),3(/,a,i4))') &
&    'Info:',&
&    '  All quantities (except nl projectors) are transfered',&
&    '  into a logarithmic grid (r(i)=A*exp[B(i-2)])...',&
&    '  Log. grid parameters: rad_step=',loggrd%rad_step,&
&    '                        log_step=',loggrd%log_step,&
&    '                        Size       =',sphmeshsz,&
&    '                        Size (core)=',coremeshsz,&
&    '                        Size (Vloc)=',vlocmeshsz
 else
  sphmeshsz=pshead%mesh_size
  coremeshsz=pshead%core_meshsz
  vlocmeshsz=pshead%vloc_meshsz
 endif
 if (loggrd%uselog) then
  nmesh=2;iprojmesh=2
  meshtp(1)=3;meshsz(1)=sphmeshsz;radstp(1)=loggrd%rad_step;logstp(1)=loggrd%log_step
  meshtp(2)=1;meshsz(2)=pshead%proj_meshsz;radstp(2)=pshead%rad_step
  if (sphmeshsz/=coremeshsz) then
   nmesh=nmesh+1;icoremesh=nmesh
   meshtp(nmesh)=3;meshsz(nmesh)=coremeshsz;radstp(nmesh)=loggrd%rad_step;logstp(nmesh)=loggrd%log_step
  endif
  if (sphmeshsz/=vlocmeshsz) then
   nmesh=nmesh+1;ivlocmesh=nmesh
   meshtp(nmesh)=3;meshsz(nmesh)=vlocmeshsz;radstp(nmesh)=loggrd%rad_step;logstp(nmesh)=loggrd%log_step
  endif
 else
  nmesh=1;iprojmesh=1
  meshtp(1)=1;meshsz(1)=sphmeshsz;radstp(1)=loggrd%rad_step
  if (sphmeshsz/=pshead%proj_meshsz) then
   nmesh=nmesh+1
   meshtp(nmesh)=1;meshsz(nmesh)=pshead%proj_meshsz;radstp(nmesh)=pshead%rad_step
  endif
  if (sphmeshsz/=coremeshsz) then
   nmesh=nmesh+1;icoremesh=nmesh
   meshtp(nmesh)=1;meshsz(nmesh)=coremeshsz;radstp(nmesh)=pshead%rad_step
  endif
  if (sphmeshsz/=vlocmeshsz) then
   nmesh=nmesh+1;ivlocmesh=nmesh
   meshtp(nmesh)=1;meshsz(nmesh)=vlocmeshsz;radstp(nmesh)=pshead%rad_step
  endif
 endif

!Open the file for writing
 open(unit=funit,file=trim(fname),form='formatted',status='unknown')

!Write the header
 call date_and_time(strdate)
 write(funit,'(a)') trim(pshead%title)
 write(funit,'(1x,f7.3,1x,f7.3,1x,a,14x,a)') &
&      pshead%atomic_charge,&
&      pshead%atomic_charge-pshead%core_charge,&
&      trim(strdate),&
&      " : zatom,zion,pspdat"
 write (funit,'(3(1x,i2)," 0 ",i5," 0.",19x,a)') &
&      pshead%pspcod,&
&      pshead%pspxc_abinit,&
&      pshead%lmax,&
&      sphmeshsz,&
&      " : pspcod,pspxc,lmax,lloc,mmax,r2well"
 write (funit,'(1x,"paw2",1x,i2,31x,a)') &
&       pshead%creatorid,&
&       " : pspfmt,creatorID"
 write (funit,'(2(1x,i2),33x,a)') &
&       pshead%basis_size,&
&       pshead%lmn_size,&
&       " : basis_size,lmn_size"
 do ib=1,pshead%basis_size
  write (funit,'(1x,i1)',ADVANCE='NO') pshead%orbitals(ib)
 enddo
 if (pshead%basis_size<20) then
  do ib=pshead%basis_size+1,20
   write (funit,'(a)',ADVANCE='NO') '  '
  enddo
 endif
 write (funit,'(a)') ": orbitals"
 write (funit,'(1x,i1,37x,a)') &
&       nmesh," : number_of_meshes"
 do ii=1,nmesh
  if (meshtp(ii)==1) then
   write (funit,'(1x,i1,1x,i1,1x,i4,1x,g18.12,15x,a,i1,a)') &
&         ii,meshtp(ii),meshsz(ii),radstp(ii),&
&         " : mesh ",ii,", type,size,rad_step[,log_step]"
  else
   write (funit,'(1x,i1,1x,i1,1x,i4,1x,g16.10,1x,g16.10,a,i1,a)') &
&         ii,meshtp(ii),meshsz(ii),radstp(ii),logstp(ii),&
&         " : mesh ",ii,", type,size,rad_step[,log_step]"
  endif
 enddo
 write (funit,'(1x,f13.10,25x,a)') &
&       pshead%rc_sph,&
&      " : r_cut(SPH)"
 if (pshead%shape_type==1) then
  write (funit,'(1x,i1,1x,i2,1x,g20.12,13x,a)') &
&         pshead%shape_type,pshead%lambda,pshead%sigma,&
&        " : shape_type,lambda,sigma"
 else
  write (funit,'(1x,i1,37x,a)') &
&        pshead%shape_type,&
&        " : shape_type"
 endif

!Write W-functions and pseudo W-functions
 nmesh=1
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== PHI ",ib,&
&      " ===== #phi(r), for phi(r)/r*Ylm)"
  write(funit,'(a)') " 1  : radial mesh index"
  if (loggrd%uselog) then
   allocate(ffit(sphmeshsz))
   call splinefit(pshead%mesh_size,pshead%rad_step,pawps%phi(:,ib),&
&                 sphmeshsz,rr_log(1:sphmeshsz),ffit)
   write(funit,'(3(1x,g23.16))') ffit
   deallocate(ffit)
  else
   write(funit,'(3(1x,g23.16))') &
&       (pawps%phi(ir,ib),ir=1,pshead%mesh_size)
  endif
 enddo
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== TPHI ",ib,&
&      " ===== #tphi(r), for tphi(r)/r*Ylm)"
  write(funit,'(a)') " 1  : radial mesh index"
  if (loggrd%uselog) then
   allocate(ffit(sphmeshsz))
   call splinefit(pshead%mesh_size,pshead%rad_step,pawps%tphi(:,ib),&
&                 sphmeshsz,rr_log(1:sphmeshsz),ffit)
   write(funit,'(3(1x,g23.16))') ffit
   deallocate(ffit)
  else
   write(funit,'(3(1x,g23.16))') &
&       (pawps%tphi(ir,ib),ir=1,pshead%mesh_size)
  endif
 enddo

!Write projectors
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== TPROJECTOR ",ib,&
&      " ===== #p(r), for p(r)/r*Ylm) )"
  write(funit,'(i2,a)') iprojmesh,"  : radial mesh index"
  write(funit,'(3(1x,g23.16))') &
&      (pawps%tproj(ir,ib),ir=1,pshead%proj_meshsz)
 enddo

!Write the core density and the pseudo core density
 write(funit,'(a)') "===== CORE_DENSITY ====="
 write(funit,'(i2,a)') icoremesh,"  : radial mesh index"
 if (loggrd%uselog) then
  allocate(ffit(coremeshsz))
  call splinefit(pshead%bigmesh_size,pshead%rad_step,pawps%coredens,&
&                coremeshsz,rr_log(1:coremeshsz),ffit)
  write(funit,'(3(1x,g23.16))') ffit
  deallocate(ffit)
 else
  write(funit,'(3(1x,g23.16))') &
&      (pawps%coredens(ir),ir=1,pshead%core_meshsz)
 endif
 write(funit,'(a)') "===== TCORE_DENSITY ====="
 write(funit,'(i2,a)') icoremesh,"  : radial mesh index"
 if (loggrd%uselog) then
  allocate(ffit(coremeshsz))
  call splinefit(pshead%bigmesh_size,pshead%rad_step,pawps%tcoredens,&
&                coremeshsz,rr_log(1:coremeshsz),ffit)
  write(funit,'(3(1x,g23.16))') ffit
  deallocate(ffit)
 else
  write(funit,'(3(1x,g23.16))') &
&      (pawps%tcoredens(ir),ir=1,pshead%core_meshsz)
 endif

!Write Dij0 and Rhoij0
 write(funit,'(a)') "===== Dij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(funit,'(100(1x,g23.16))') (pawps%dij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo
 write(funit,'(a)') "===== Rhoij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(funit,'(100(1x,g23.16))') (pawps%rhoij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo

!Write VHntZC=Vloc
 write(funit,'(a)') "===== VHntZC (Vloc(r)) ====="
 write(funit,'(i2,a)') ivlocmesh,"  : radial mesh index"
 if (loggrd%uselog) then
  allocate(ffit(vlocmeshsz))
  call splinefit(pshead%bigmesh_size,pshead%rad_step,pawps%vhtnzc,&
&                vlocmeshsz,rr_log(1:vlocmeshsz),ffit)
  write(funit,'(3(1x,g23.16))') ffit
  deallocate(ffit)
 else
  write(funit,'(3(1x,g23.16))') &
&      (pawps%vhtnzc(ir),ir=1,pshead%vloc_meshsz)
 endif

!Close the file and end
 close(funit)
 if (loggrd%uselog) deallocate(rr_log)

 end subroutine
!!***
